@@ -127,13 +127,20 @@ SYMBOL_EDIT_FRAME::SYMBOL_EDIT_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
     LoadSettings( m_settings );
 
     m_libMgr = new SYMBOL_LIBRARY_MANAGER( *this );
+    bool loadingCancelled = false;
 
-    // Preload libraries before using SyncLibraries the first time, as the preload is threaded
-    WX_PROGRESS_REPORTER reporter( this, _( "Loading Symbol Libraries" ),
-                                   m_libMgr->GetLibraryCount(), true );
-    m_libMgr->Preload( reporter );
+    {
+        // Preload libraries before using SyncLibraries the first time, as the preload is
+        // multi-threaded
+        WX_PROGRESS_REPORTER reporter( this, _( "Loading Symbol Libraries" ),
+                                       m_libMgr->GetLibraryCount(), true );
+        m_libMgr->Preload( reporter );
 
-    SyncLibraries( false );
+        loadingCancelled = reporter.IsCancelled();
+        wxSafeYield();
+    }
+
+    SyncLibraries( false, loadingCancelled );
     m_treePane = new SYMBOL_TREE_PANE( this, m_libMgr );
 
     resolveCanvasType();
@@ -168,28 +175,28 @@ SYMBOL_EDIT_FRAME::SYMBOL_EDIT_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
     m_auimgr.SetManagedWindow( this );
 
     CreateInfoBar();
-    m_auimgr.AddPane( m_mainToolBar, EDA_PANE().HToolbar().Name( "MainToolbar" )
+    m_auimgr.AddPane( m_mainToolBar, EDA_PANE().HToolbar().Name( wxT( "MainToolbar" ) )
                       .Top().Layer( 6 ) );
-    m_auimgr.AddPane( m_messagePanel, EDA_PANE().Messages().Name( "MsgPanel" )
+    m_auimgr.AddPane( m_messagePanel, EDA_PANE().Messages().Name( wxT( "MsgPanel" ) )
                       .Bottom().Layer( 6 ) );
 
-    m_auimgr.AddPane( m_optionsToolBar, EDA_PANE().VToolbar().Name( "OptToolbar" )
+    m_auimgr.AddPane( m_optionsToolBar, EDA_PANE().VToolbar().Name( wxT( "OptToolbar" ) )
                       .Left().Layer( 3 ) );
-    m_auimgr.AddPane( m_treePane, EDA_PANE().Palette().Name( "SymbolTree" )
+    m_auimgr.AddPane( m_treePane, EDA_PANE().Palette().Name( wxT( "SymbolTree" ) )
                       .Left().Layer( 2 )
                       .Caption( _( "Libraries" ) )
                       .MinSize( 250, -1 ).BestSize( 250, -1 ) );
-    m_auimgr.AddPane( m_drawToolBar, EDA_PANE().VToolbar().Name( "ToolsToolbar" )
+    m_auimgr.AddPane( m_drawToolBar, EDA_PANE().VToolbar().Name( wxT( "ToolsToolbar" ) )
                       .Right().Layer( 2 ) );
 
-    m_auimgr.AddPane( GetCanvas(), wxAuiPaneInfo().Name( "DrawFrame" )
+    m_auimgr.AddPane( GetCanvas(), wxAuiPaneInfo().Name( wxT( "DrawFrame" ) )
                       .CentrePane() );
 
     FinishAUIInitialization();
 
     if( m_settings->m_LibWidth > 0 )
     {
-        wxAuiPaneInfo& treePane = m_auimgr.GetPane( "SymbolTree" );
+        wxAuiPaneInfo& treePane = m_auimgr.GetPane( wxT( "SymbolTree" ) );
 
         // wxAUI hack: force width by setting MinSize() and then Fixed()
         // thanks to ZenJu http://trac.wxwidgets.org/ticket/13180
@@ -235,6 +242,9 @@ SYMBOL_EDIT_FRAME::SYMBOL_EDIT_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
 
     // Ensure the window is on top
     Raise();
+
+    if( loadingCancelled )
+        ShowInfoBarWarning( _( "Symbol library loading was cancelled by user." ) );
 }
 
 
@@ -253,6 +263,7 @@ SYMBOL_EDIT_FRAME::~SYMBOL_EDIT_FRAME()
         delete screen;
         m_isSymbolFromSchematic = false;
     }
+
     // current screen is destroyed in EDA_DRAW_FRAME
     SetScreen( m_dummyScreen );
 
@@ -265,7 +276,7 @@ SYMBOL_EDIT_FRAME::~SYMBOL_EDIT_FRAME()
 
 void SYMBOL_EDIT_FRAME::LoadSettings( APP_SETTINGS_BASE* aCfg )
 {
-    wxCHECK_RET( m_settings, "Call to SYMBOL_EDIT_FRAME::LoadSettings with null m_boardAdapter" );
+    wxCHECK_RET( m_settings, wxT( "Call to SYMBOL_EDIT_FRAME::LoadSettings with null m_boardAdapter" ) );
 
     SCH_BASE_FRAME::LoadSettings( GetSettings() );
 
@@ -280,7 +291,7 @@ void SYMBOL_EDIT_FRAME::LoadSettings( APP_SETTINGS_BASE* aCfg )
 
 void SYMBOL_EDIT_FRAME::SaveSettings( APP_SETTINGS_BASE* aCfg )
 {
-    wxCHECK_RET( m_settings, "Call to SYMBOL_EDIT_FRAME::LoadSettings with null m_boardAdapter" );
+    wxCHECK_RET( m_settings, wxT( "Call to SYMBOL_EDIT_FRAME::LoadSettings with null m_boardAdapter" ) );
 
     GetGalDisplayOptions().m_axesEnabled = true;
 
@@ -365,10 +376,11 @@ void SYMBOL_EDIT_FRAME::setupUIConditions()
                 return IsSymbolEditable() && !IsSymbolAlias();
             };
 
-    auto libModifiedCondition =
+    auto symbolModifiedCondition =
             [this]( const SELECTION& sel )
             {
-                return m_libMgr->HasModifications();
+                return m_libMgr->IsSymbolModified( GetTargetLibId().GetLibItemName(),
+                                                   GetTargetLibId().GetLibNickname() );
             };
 
     auto libSelectedCondition =
@@ -398,7 +410,7 @@ void SYMBOL_EDIT_FRAME::setupUIConditions()
 
     mgr->SetConditions( ACTIONS::undo,              ENABLE( haveSymbolCond && cond.UndoAvailable() ) );
     mgr->SetConditions( ACTIONS::redo,              ENABLE( haveSymbolCond && cond.RedoAvailable() ) );
-    mgr->SetConditions( ACTIONS::revert,            ENABLE( haveSymbolCond && libModifiedCondition ) );
+    mgr->SetConditions( ACTIONS::revert,            ENABLE( symbolModifiedCondition ) );
 
     mgr->SetConditions( ACTIONS::toggleGrid,        CHECK( cond.GridVisible() ) );
     mgr->SetConditions( ACTIONS::toggleCursorStyle, CHECK( cond.FullscreenCursor() ) );
@@ -803,11 +815,13 @@ SYMBOL_LIBRARY_MANAGER& SYMBOL_EDIT_FRAME::GetLibManager()
 void SYMBOL_EDIT_FRAME::OnModify()
 {
     GetScreen()->SetContentModified();
-    storeCurrentSymbol();
+
+    if( !IsSymbolFromSchematic() )
+        storeCurrentSymbol();
 
     m_treePane->GetLibTree()->RefreshLibTree();
 
-    if( !GetTitle().StartsWith( "*" ) )
+    if( !GetTitle().StartsWith( wxT( "*" ) ) )
         updateTitle();
 }
 
@@ -922,7 +936,8 @@ wxString SYMBOL_EDIT_FRAME::getTargetLib() const
 }
 
 
-void SYMBOL_EDIT_FRAME::SyncLibraries( bool aShowProgress, const wxString& aForceRefresh )
+void SYMBOL_EDIT_FRAME::SyncLibraries( bool aShowProgress, bool aPreloadCancelled,
+                                       const wxString& aForceRefresh )
 {
     LIB_ID selected;
 
@@ -941,7 +956,7 @@ void SYMBOL_EDIT_FRAME::SyncLibraries( bool aShowProgress, const wxString& aForc
                                                                     libName ) );
                 } );
     }
-    else
+    else if( !aPreloadCancelled )
     {
         m_libMgr->Sync( aForceRefresh,
                 [&]( int progress, int max, const wxString& libName )
@@ -968,7 +983,7 @@ void SYMBOL_EDIT_FRAME::SyncLibraries( bool aShowProgress, const wxString& aForc
         // Try to select the parent library, in case the symbol is not found
         if( !found && selected.IsValid() )
         {
-            selected.SetLibItemName( "" );
+            selected.SetLibItemName( wxEmptyString );
             found = m_libMgr->GetAdapter()->FindItem( selected );
 
             if( found )
@@ -1105,6 +1120,9 @@ void SYMBOL_EDIT_FRAME::CommonSettingsChanged( bool aEnvVarsChanged, bool aTextV
     GetCanvas()->GetGAL()->SetAxesColor( m_colorSettings->GetColor( LAYER_SCHEMATIC_GRID_AXES ) );
     GetCanvas()->GetGAL()->DrawGrid();
 
+    GetCanvas()->GetView()->UpdateAllItems( KIGFX::ALL );
+    GetCanvas()->Refresh();
+
     RecreateToolbars();
 
     if( aEnvVarsChanged )
@@ -1125,6 +1143,14 @@ void SYMBOL_EDIT_FRAME::ShowChangedLanguage()
 
     // status bar
     UpdateMsgPanel();
+
+    if( GetRenderSettings()->m_ShowPinsElectricalType )
+    {
+        GetCanvas()->GetView()->UpdateAllItems( KIGFX::ALL );
+        GetCanvas()->Refresh();
+    }
+
+    updateTitle();
 }
 
 
@@ -1144,6 +1170,7 @@ void SYMBOL_EDIT_FRAME::RebuildView()
     GetCanvas()->GetView()->HideDrawingSheet();
     GetCanvas()->GetView()->ClearHiddenFlags();
 
+    GetCanvas()->GetView()->UpdateAllItems( KIGFX::ALL );
     GetCanvas()->Refresh();
 }
 
@@ -1375,7 +1402,7 @@ bool SYMBOL_EDIT_FRAME::addLibTableEntry( const wxString& aLibFile, TABLE_SCOPE
 
         while( libTable->HasLibrary( libNickname ) )
         {
-            tmp.Printf( "%s%d", fn.GetName(), suffix );
+            tmp.Printf( wxT( "%s%d" ), fn.GetName(), suffix );
             libNickname = tmp;
             suffix += 1;
         }
@@ -1449,7 +1476,7 @@ bool SYMBOL_EDIT_FRAME::replaceLibTableEntry( const wxString& aLibNickname,
     wxString normalizedPath = NormalizePath( aLibFile, &envVars, projectPath );
 
     row->SetFullURI( normalizedPath );
-    row->SetType( "KiCad" );
+    row->SetType( wxT( "KiCad" ) );
 
     try
     {
