@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2019 CERN
- * Copyright (C) 2019-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2019-2022 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -668,10 +668,11 @@ int SCH_EDIT_TOOL::Rotate( const TOOL_EVENT& aEvent )
     }
     else
     {
+        EE_SELECTION selectionCopy = selection;
+
         if( selection.IsHover() )
             m_toolMgr->RunAction( EE_ACTIONS::clearSelection, true );
 
-        EE_SELECTION selectionCopy = selection;
         m_toolMgr->RunAction( EE_ACTIONS::trimOverlappingWires, true, &selectionCopy );
         m_toolMgr->RunAction( EE_ACTIONS::addNeededJunctions, true, &selectionCopy );
 
@@ -1394,6 +1395,13 @@ int SCH_EDIT_TOOL::Properties( const TOOL_EVENT& aEvent )
             auto editor = (SYMBOL_EDIT_FRAME*) m_frame->Kiway().Player( FRAME_SCH_SYMBOL_EDITOR,
                                                                         true );
 
+            if( wxWindow* blocking_win = editor->Kiway().GetBlockingDialog() )
+                blocking_win->Close( true );
+
+            // The broken library symbol link indicator cannot be edited.
+            if( symbol->IsMissingLibSymbol() )
+                return 0;
+
             editor->LoadSymbolFromSchematic( symbol );
 
             editor->Show( true );
@@ -1404,6 +1412,9 @@ int SCH_EDIT_TOOL::Properties( const TOOL_EVENT& aEvent )
             auto editor = (SYMBOL_EDIT_FRAME*) m_frame->Kiway().Player( FRAME_SCH_SYMBOL_EDITOR,
                                                                         true );
 
+            if( wxWindow* blocking_win = editor->Kiway().GetBlockingDialog() )
+                blocking_win->Close( true );
+
             editor->LoadSymbol( symbol->GetLibId(), symbol->GetUnit(), symbol->GetConvert() );
 
             editor->Show( true );
