@@ -827,29 +827,37 @@ void PCB_PAINTER::draw( const PCB_VIA* aVia, int aLayer )
     default:                    wxASSERT( false );                                           break;
     }
 
+    m_gal->SetStrokeColor( color );
+    m_gal->SetFillColor( color );
+    m_gal->SetIsStroke( true );
+    m_gal->SetIsFill( false );
+
     if( sketchMode )
-    {
-        // Outline mode
-        m_gal->SetIsStroke( true );
-        m_gal->SetIsFill( false );
         m_gal->SetLineWidth( m_pcbSettings.m_outlineWidth );
-        m_gal->SetStrokeColor( color );
-    }
-    else
-    {
-        // Filled mode
-        m_gal->SetIsFill( true );
-        m_gal->SetIsStroke( false );
-        m_gal->SetFillColor( color );
-    }
 
     if( aLayer == LAYER_VIA_HOLES )
     {
+        m_gal->SetIsStroke( false );
+        m_gal->SetIsFill( true );
         m_gal->DrawCircle( center, getDrillSize( aVia ) / 2.0 );
     }
-    else if( aLayer == LAYER_VIA_THROUGH || m_pcbSettings.GetDrawIndividualViaLayers() )
+    else if( aLayer == LAYER_VIA_THROUGH || m_pcbSettings.IsPrinting() )
     {
-        m_gal->DrawCircle( center, aVia->GetWidth() / 2.0 );
+        int    annular_width = ( aVia->GetWidth() - getDrillSize( aVia ) ) / 2.0;
+        double radius = aVia->GetWidth() / 2.0;
+        bool   draw = aLayer == LAYER_VIA_THROUGH;
+
+        if( m_pcbSettings.IsPrinting() )
+            draw = aVia->FlashLayer( m_pcbSettings.GetPrintLayers() );
+
+        if( !sketchMode )
+        {
+            m_gal->SetLineWidth( annular_width );
+            radius -= annular_width / 2.0;
+        }
+
+        if( draw )
+            m_gal->DrawCircle( center, radius );
     }
     else if( aLayer == LAYER_VIA_BBLIND || aLayer == LAYER_VIA_MICROVIA )
     {
@@ -861,7 +869,11 @@ void PCB_PAINTER::draw( const PCB_VIA* aVia, int aLayer )
         double radius =  aVia->GetWidth() / 2.0;
 
         if( !sketchMode )
+        {
+            m_gal->SetIsFill( true );
+            m_gal->SetIsStroke( false );
             m_gal->SetLineWidth( ( aVia->GetWidth() - aVia->GetDrillValue() ) / 2.0 );
+        }
 
         m_gal->DrawArc( center, radius, M_PI * -0.375, M_PI * 0.375 );
         m_gal->DrawArc( center, radius, M_PI * 0.625, M_PI * 1.375 );
@@ -1235,6 +1247,9 @@ void PCB_PAINTER::draw( const PAD* aPad, int aLayer )
                 {
                     const SHAPE_SIMPLE* poly = static_cast<const SHAPE_SIMPLE*>( shape );
 
+                    if( poly->PointCount() < 2 )     // Careful of empty pads
+                        break;
+
                     if( margin.x < 0 )  // The poly shape must be deflated
                     {
                         int numSegs = GetArcToSegmentCount( -margin.x, m_maxError, 360.0 );
@@ -1303,6 +1318,9 @@ void PCB_PAINTER::draw( const PAD* aPad, int aLayer )
 
         if( flashActiveLayer || aPad->GetDrillSize().x )
         {
+            if( aPad->GetAttribute() == PAD_ATTRIB::NPTH )
+                color = m_pcbSettings.GetLayerColor( LAYER_NON_PLATEDHOLES );
+
             m_gal->SetLineWidth( m_pcbSettings.m_outlineWidth );
             m_gal->SetIsStroke( true );
             m_gal->SetIsFill( false );
@@ -1332,7 +1350,9 @@ void PCB_PAINTER::draw( const PAD* aPad, int aLayer )
                     // Use ERROR_INSIDE because it avoids Clipper and is therefore much faster.
                     aPad->TransformShapeWithClearanceToPolygon( polySet, ToLAYER_ID( aLayer ),
                                                                 clearance, m_maxError, ERROR_INSIDE );
-                    m_gal->DrawPolygon( polySet );
+
+                    if( polySet.Outline( 0 ).PointCount() > 2 )     // Careful of empty pads
+                        m_gal->DrawPolygon( polySet );
                 }
             }
             else if( aPad->GetEffectiveHoleShape() && clearance > 0 )
