@@ -49,6 +49,7 @@ using namespace std::placeholders;
 #include <tools/pcb_actions.h>
 #include <tools/pcb_selection_tool.h>
 #include <tools/pcb_grid_helper.h>
+#include <string_utils.h>
 
 #include "router_tool.h"
 #include "pns_segment.h"
@@ -170,7 +171,8 @@ static const TOOL_ACTION ACT_SwitchCornerMode( "pcbnew.InteractiveRouter.SwitchR
 
 ROUTER_TOOL::ROUTER_TOOL() :
         TOOL_BASE( "pcbnew.InteractiveRouter" ),
-        m_lastTargetLayer( UNDEFINED_LAYER )
+        m_lastTargetLayer( UNDEFINED_LAYER ),
+                m_inRouterTool( false )
 {
 }
 
@@ -615,6 +617,54 @@ void ROUTER_TOOL::switchLayerOnViaPlacement()
 
     m_router->SwitchLayer( *newLayer );
     m_lastTargetLayer = *newLayer;
+
+    updateSizesAfterLayerSwitch( ToLAYER_ID( *newLayer ) );
+}
+
+
+void ROUTER_TOOL::updateSizesAfterLayerSwitch( PCB_LAYER_ID targetLayer )
+{
+    std::vector<int> nets = m_router->GetCurrentNets();
+
+    PNS::SIZES_SETTINGS          sizes     = m_router->Sizes();
+    BOARD_DESIGN_SETTINGS&       bds       = board()->GetDesignSettings();
+    std::shared_ptr<DRC_ENGINE>& drcEngine = bds.m_DRCEngine;
+    DRC_CONSTRAINT               constraint;
+
+    PCB_TRACK dummyTrack( board() );
+    dummyTrack.SetLayer( targetLayer );
+    dummyTrack.SetNetCode( nets.empty() ? 0 : nets[0] );
+
+    if( bds.UseNetClassTrack() || !sizes.TrackWidthIsExplicit() )
+    {
+        constraint = drcEngine->EvalRules( TRACK_WIDTH_CONSTRAINT, &dummyTrack, nullptr,
+                                           targetLayer );
+
+        if( !constraint.IsNull() )
+            sizes.SetTrackWidth( std::max( bds.m_TrackMinWidth, constraint.m_Value.Opt() ) );
+    }
+
+    if( nets.size() >= 2 && ( bds.UseNetClassDiffPair() || !sizes.TrackWidthIsExplicit() ) )
+    {
+        PCB_TRACK dummyTrackB( board() );
+        dummyTrackB.SetLayer( targetLayer );
+        dummyTrackB.SetNetCode( nets[1] );
+
+        constraint = drcEngine->EvalRules( TRACK_WIDTH_CONSTRAINT, &dummyTrack, &dummyTrackB,
+                                           targetLayer );
+
+        if( !constraint.IsNull() )
+            sizes.SetDiffPairWidth( std::max( bds.m_TrackMinWidth, constraint.m_Value.Opt() ) );
+
+        constraint = drcEngine->EvalRules( DIFF_PAIR_GAP_CONSTRAINT, &dummyTrack, &dummyTrackB,
+                                           targetLayer );
+
+        if( !constraint.IsNull() )
+            sizes.SetDiffPairGap( std::max( bds.m_MinClearance, constraint.m_Value.Opt() ) );
+    }
+
+    m_router->UpdateSizes( sizes );
+    frame()->UpdateMsgPanel();
 }
 
 
@@ -782,6 +832,7 @@ int ROUTER_TOOL::handleLayerSwitch( const TOOL_EVENT& aEvent, bool aForceVia )
 
         if( !aForceVia && m_router && m_router->SwitchLayer( targetLayer ) )
         {
+            updateSizesAfterLayerSwitch( targetLayer );
             updateEndItem( aEvent );
             m_router->Move( m_endSnapPoint, m_endItem );        // refresh
             return 0;
@@ -1348,6 +1399,11 @@ void ROUTER_TOOL::breakTrack()
 
 int ROUTER_TOOL::MainLoop( const TOOL_EVENT& aEvent )
 {
+    if( m_inRouterTool )
+        return 0;
+
+    REENTRANCY_GUARD guard( &m_inRouterTool );
+
     PNS::ROUTER_MODE mode = aEvent.Parameter<PNS::ROUTER_MODE>();
     PCB_EDIT_FRAME*  frame = getEditFrame<PCB_EDIT_FRAME>();
     VIEW_CONTROLS*   controls = getViewControls();
@@ -1459,6 +1515,7 @@ int ROUTER_TOOL::MainLoop( const TOOL_EVENT& aEvent )
         else if( evt->IsAction( &PCB_ACTIONS::layerChanged ) )
         {
             m_router->SwitchLayer( frame->GetActiveLayer() );
+            updateSizesAfterLayerSwitch( frame->GetActiveLayer() );
             updateStartItem( *evt );
         }
         else if( evt->IsKeyPressed() )
@@ -1705,10 +1762,10 @@ int ROUTER_TOOL::InlineDrag( const TOOL_EVENT& aEvent )
         item->SetLocked( false );
     }
 
+    m_toolMgr->RunAction( PCB_ACTIONS::selectionClear, true );
+
     Activate();
 
-    m_toolMgr->RunAction( PCB_ACTIONS::selectionClear, true );
-    m_router->SyncWorld();
     m_startItem = nullptr;
 
     PNS::ITEM* startItem = nullptr;
@@ -1985,10 +2042,10 @@ int ROUTER_TOOL::InlineBreakTrack( const TOOL_EVENT& aEvent )
     if( item->Type() != PCB_TRACE_T )
         return 0;
 
+    m_toolMgr->RunAction( PCB_ACTIONS::selectionClear, true );
+
     Activate();
 
-    m_toolMgr->RunAction( PCB_ACTIONS::selectionClear, true );
-    m_router->SyncWorld();
     m_startItem = m_router->GetWorld()->FindItemByParent( item );
 
     TOOL_MANAGER* toolManager = frame()->GetToolManager();
@@ -2085,22 +2142,53 @@ void ROUTER_TOOL::UpdateMessagePanel()
     PNS::SIZES_SETTINGS sizes( m_router->Sizes() );
     PNS::RULE_RESOLVER* resolver   = m_iface->GetRuleResolver();
     bool                isDiffPair = m_router->Mode() == PNS::ROUTER_MODE::PNS_MODE_ROUTE_DIFF_PAIR;
+    std::vector<int>    nets = m_router->GetCurrentNets();
+    wxString            description;
+    wxString            secondary;
 
-    if( m_startItem && m_startItem->Net() > 0 )
+    if( isDiffPair )
     {
-        wxString description = isDiffPair ? _( "Routing Diff Pair: %s" ) : _( "Routing Track: %s" );
+        wxASSERT( nets.size() >= 2 );
 
-        NETINFO_ITEM* netInfo = board()->FindNet( m_startItem->Net() );
-        wxASSERT( netInfo );
+        NETINFO_ITEM* netA = board()->FindNet( nets[0] );
+        NETINFO_ITEM* netB = board()->FindNet( nets[1] );
+        wxASSERT( netA );
+        wxASSERT( netB );
 
-        items.emplace_back( wxString::Format( description, netInfo->GetNetname() ),
-                            wxString::Format( _( "Net Class: %s" ), netInfo->GetNetClassName() ) );
+        description = wxString::Format( _( "Routing Diff Pair: %s" ),
+                                        netA->GetNetname() + wxT( ", " ) + netB->GetNetname() );
+
+        wxString  netclass;
+        NETCLASS* netclassA = netA->GetNetClass();
+        NETCLASS* netclassB = netB->GetNetClass();
+
+        if( netclassA == netclassB )
+            netclass = netclassA->GetName();
+        else
+            netclass = netclassA->GetName() + wxT( ", " ) + netclassB->GetName();
+
+        secondary = wxString::Format( _( "Net Class: %s" ),
+                                      UnescapeString( netclass ) );
+    }
+    else if( !nets.empty() )
+    {
+        NETINFO_ITEM* net = board()->FindNet( nets[0] );
+        wxASSERT( net );
+
+        description = wxString::Format( _( "Routing Track: %s" ),
+                                        net->GetNetname() );
+
+        secondary = wxString::Format( _( "Net Class: %s" ),
+                                      UnescapeString( net->GetNetClass()->GetName() ) );
     }
     else
     {
-        items.emplace_back( _( "Routing Track" ), _( "(no net)" ) );
+        description = _( "Routing Track" );
+        secondary = _( "(no net)" );
     }
 
+    items.emplace_back( description, secondary );
+
     wxString cornerMode;
 
     if( m_router->Settings().GetFreeAngleMode() )
