@@ -2376,7 +2376,7 @@ int DRAWING_TOOL::DrawVia( const TOOL_EVENT& aEvent )
             {
                 BOARD_ITEM* item = static_cast<BOARD_ITEM*>( it.first );
 
-                if( !(item->GetLayerSet() & lset ).any() )
+                if( !( item->GetLayerSet() & lset ).any() )
                     continue;
 
                 if( PCB_TRACK* track = dyn_cast<PCB_TRACK*>( item ) )
@@ -2412,25 +2412,26 @@ int DRAWING_TOOL::DrawVia( const TOOL_EVENT& aEvent )
             if( DRC_ENGINE::IsNetTie( aOther ) )
                 return false;
 
-            DRC_CONSTRAINT constraint;
+            DRC_CONSTRAINT        constraint;
+            int                   clearance;
+            BOARD_CONNECTED_ITEM* connectedItem = dynamic_cast<BOARD_CONNECTED_ITEM*>( aOther );
+            ZONE*                 zone = dynamic_cast<ZONE*>( aOther );
 
-            if( ( aOther->Type() == PCB_ZONE_T || aOther->Type() == PCB_FP_ZONE_T )
-                    && static_cast<ZONE*>( aOther )->GetIsRuleArea() )
+            if( zone && zone->GetIsRuleArea() )
             {
-                ZONE* ruleArea = static_cast<ZONE*>( aOther );
-
-                if( ruleArea->GetDoNotAllowVias() )
-                    return ruleArea->Outline()->Collide( aVia->GetPosition(), aVia->GetWidth() / 2 );
+                if( zone->GetDoNotAllowVias() )
+                    return zone->Outline()->Collide( aVia->GetPosition(), aVia->GetWidth() / 2 );
 
                 return false;
             }
 
-            BOARD_CONNECTED_ITEM* cItem = dynamic_cast<BOARD_CONNECTED_ITEM*>( aOther );
+            if( connectedItem )
+            {
+                int connectedItemNet = connectedItem->GetNetCode();
 
-            if( cItem && cItem->GetNetCode() == aVia->GetNetCode() )
-                return false;
-
-            int  clearance;
+                if( connectedItemNet == 0 || connectedItemNet == aVia->GetNetCode() )
+                    return false;
+            }
 
             for( PCB_LAYER_ID layer : aOther->GetLayerSet().Seq() )
             {
@@ -2576,7 +2577,7 @@ int DRAWING_TOOL::DrawVia( const TOOL_EVENT& aEvent )
             {
                 for( ZONE* z : m_board->Zones() )
                 {
-                    if( z->IsOnLayer( m_frame->GetActiveLayer() ) )
+                    if( z->IsOnLayer( layer ) )
                     {
                         if( z->HitTestFilledArea( layer, position ) )
                             return z->GetNetCode();
